# Copyright(c) 2002-2005 Association of Universities for Research in Astronomy, Inc.

procedure gsappwave (inimages)

# Wavelength calibrate GMOS spectra
#
# Version   Feb 28, 2002  BM,IJ   v1.3 release
#           Sept 20, 2002         v1.4 release
#           Oct 14, 2002  IJ      don't use ijk from within script
#           Mar 20, 2003  BM      pixel scale for both instruments
#           May 9, 2003   IJ      change in instrument logic, support for old GMOS-N data
#           Aug 26, 2003  KL      IRAF2.12 - new parameter, addonly, in hedit
#           Mar 26, 2004  BM      2 filter support like gscut/gfextract

string inimages   {"",prompt="Input images"}
string gratingdb  {"gmos$data/GMOSgratings.dat",prompt="Gratings database file"}
string filterdb   {"gmos$data/GMOSfilters.dat",prompt="Filters database file"}
string key_dispaxis {"DISPAXIS",prompt="Keyword for dispersion axis"}
int    dispaxis   {1,min=1,max=2,prompt="Dispersion axis"}
string logfile    {"",prompt="Logfile"}
bool   verbose    {yes,prompt="Verbose output?"}
int    status     {0,prompt="Exit status (0=good)"}
struct *scanfile  {"",prompt="Internal use only"}

begin

string l_inimages,l_logfile, l_filterdb, l_gratingdb, l_linelist
string l_grating, l_filter[2], l_key_dispaxis
string obsmode, infiles, inlist, img
string gname, fname, ffile, gscut
real l_wave1, wave1, wave2, fwave1, fwave2, gwave1, gwave2, wmin, wmax
real cwave,gblaze,gR,gcoverage,nmppx,scale[2],pi,tilt,gtilt,a,greq
real refpix,pixcwave,wavoffset
real wmn1,wmn2,wmx1,wmx2
int l_xbin,l_ybin,grule,speclen,width,center,xccd[2],yccd[2],slitsep
int nexten, l_dispaxis, nxpix,nypix, n_i, inst
bool l_verbose
struct sdate

# Query parameters
l_inimages=inimages
l_logfile=logfile
l_gratingdb=gratingdb ; l_filterdb=filterdb
l_key_dispaxis=key_dispaxis ; l_dispaxis=dispaxis
l_verbose=verbose
status=0
pi=3.14159265
scale[1]=0.0727
scale[2]=0.073

# Keep imgets parameters from changing by outside world
cache("imgets", "hedit", "fparse")

# Test the logfile:
if (l_logfile == "" || stridx(" ",l_logfile)>0) {
    l_logfile = gmos.logfile
    if (l_logfile == "" || stridx(" ",l_logfile)>0) {
        l_logfile = "gmos.log"
        printlog("WARNING - GSAPPWAVE: Both gsappwave.logfile and gmos.logfile fields are empty",l_logfile,yes)
        printlog("                    Using default file gmos.log",l_logfile,yes)
    }
}

# Temporary files
infiles=mktemp("tmpinfiles")

# Start logging to file
if (l_logfile == "STDOUT") {
   l_logfile = ""
   l_verbose=yes
}

date | scan(sdate)
printlog("--------------------------------------------------------------------------------",l_logfile,verbose=l_verbose)
printlog("GSAPPWAVE -- "//sdate,l_logfile,verbose=l_verbose)
printlog(" ",l_logfile,verbose=l_verbose)
printlog("inimages = "//l_inimages,l_logfile,verbose=l_verbose)

#check that there are input files
if (l_inimages == "" || l_inimages == " ") {
  printlog("ERROR - GSAPPWAVE: input files not specified",l_logfile,yes)
  goto error
}

# check existence of list file
if (substr(l_inimages,1,1) == "@") {
    inlist=substr(l_inimages,2,strlen(l_inimages))
    if(!access(inlist)) {
        printlog("ERROR - GSWAPPWAVE: Input list "//inlist//" not found",l_logfile,verbose+)
        goto error
    }
}

# Loop over inimages
files(l_inimages,sort-, > infiles)
scanfile=infiles
while (fscan(scanfile, img) != EOF) {
    # Check existence of file
    if (!imaccess(img)) {
        printlog("ERROR - GSAPPWAVE: "//img//" does not exist.",l_logfile,yes)
        goto error
    }

    # get header information
    # Which instrument?
    imgets(img//"[0]","INSTRUME", >>& "dev$null")
    if (imgets.value == "0") {
            printlog ("ERROR - GSAPPWAVE: Instrument keyword not found.",l_logfile, verbose+)
            goto error
    }
    inst=1 # default is GMOS-N, support for old data
    if (imgets.value == "GMOS-S") {
        inst=2
    }

    imgets(img//"[0]","GRATING")
    l_grating=imgets.value

    imgets(img//"[sci,1]","CCDSUM")
    l_xbin=int(substr(imgets.value,1,1))
    l_ybin=int(substr(imgets.value,3,3))

    imgets(img//"[0]","FILTER1")
    l_filter[1]=imgets.value

    imgets(img//"[0]","FILTER2")
    l_filter[2]=imgets.value

    imgets(img//"[0]","GRWLEN")
    cwave=real(imgets.value)

    imgets(img//"[0]","GRTILT")
    tilt=real(imgets.value)*pi/180.

    imgets(img//"[0]","NSCIEXT")
    nexten=int(imgets.value)

    imgets(img//"[0]","OBSMODE")
    obsmode=imgets.value

    imgets(img//"[0]","GSCUT", >>& "dev$null")
    gscut=imgets.value

    imgets(img//"[sci,1]","i_naxis1")
    nxpix=int(imgets.value)

    imgets(img//"[sci,1]","i_naxis2")
    nypix=int(imgets.value)

    # get grating information
    match(l_grating,l_gratingdb,stop-,print+,meta+) | scan(gname,grule,gblaze,gR,gcoverage,gwave1,gwave2,wavoffset)

    print(((cwave*grule)/1.e6)) | interp("gmos$data/gratingeq.dat","STDIN",int_mode="spline",curve_gen-) | scan(greq,gtilt)

    gtilt=gtilt*pi/180.
    a=sin(gtilt+0.872665)/sin(gtilt)
    gR=206265.*greq/(0.5*81.0*sin(gtilt))
    nmppx=a*scale[inst]*real(l_xbin)*cwave*81.0*sin(gtilt)/(206265.*greq)
    gcoverage=nmppx*real(nxpix)

    wave1=cwave-gcoverage/2.
    wave2=wave1+gcoverage

    printlog("Grating: "//gname,l_logfile,l_verbose)
    printlog("Resolution (0.5'' slit): "//gR,l_logfile,l_verbose)
    printlog("Anamorphic factor: "//a,l_logfile,l_verbose)
    printlog("Grating tilt (header): "//(tilt*180./pi),l_logfile,l_verbose)
    printlog("Calculated tilt: "//(gtilt*180./pi),l_logfile,l_verbose)
    printlog("nm/pix = "//nmppx,l_logfile,l_verbose)

    # get filter information
    fwave1=0.0 ; wmn1=0.0 ; wmn2=0.0
    fwave2=9999.0 ; wmx1=99999.0 ; wmx2=99999.0
    if (l_filter[1] != "" && substr(l_filter[1],1,4) != "open" ) 
        match(l_filter[1],l_filterdb,stop-,print+,meta+) | \
            scan(fname,wmn1,wmx1,ffile)
    
    if (l_filter[2] != "" && substr(l_filter[2],1,4) != "open")
        match(l_filter[2],l_filterdb,stop-,print+,meta+) | \
            scan(fname,wmn2,wmx2,ffile)

    if (wmn1 > wmn2) {
        fwave1=wmn1
    } else {
        fwave1=wmn2
    }
    if (wmx1 < wmx2) {
        fwave2=wmx1
    } else {
        fwave2=wmx2
    }

    # determine whether filter or grating limits wavelength coverage
    if (fwave1 > wave1) {
        wave1 = fwave1
    }
    if (fwave2 < wave2) {
        wave2 = fwave2
    }
    # and red limit of CCD
    if (wave2 > 1050.) {
        wave2 = 1050.
    }

    # length of spectrum in pixels and reference pixel (of cwave)
    if (gscut == "0") {
        speclen=nxpix
        refpix=real(nxpix)/2.+wavoffset/nmppx
    } else {
        speclen=nint((wave2-wave1)/nmppx)
        refpix=real(speclen)-(cwave-wavoffset-wave1)/nmppx    
    }

    printlog("Pixel of central wavelength: "//refpix,l_logfile,l_verbose)
    printlog("Filter1: "//l_filter[1],l_logfile,l_verbose)
    printlog("Filter2: "//l_filter[2],l_logfile,l_verbose)
    printlog("Wavelength coverage: "//wave1//" "//wave2,l_logfile,l_verbose)
    printlog("Spectrum length = "//speclen,l_logfile,l_verbose)
    
    # Loop over extensions
    for (n_i=1; n_i<=nexten; n_i+=1) {
        printlog("Calibrating group: "//n_i,l_logfile,l_verbose)
        if (imaccess(img//"[SCI,"//n_i//"]")) {
            hedit(img//"[SCI,"//n_i//"]","WAT1_001",
	        "wtype=linear label=Wavelength units=Angstroms",add-,addonly-,
		delete-,verify-,show-,update+)
            imgets(img//"[SCI,"//n_i//"]","REFPIX1", >>& "dev$null")
            if (imgets.value!="0") {
                refpix=real(imgets.value)
            }
            hedit(img//"[SCI,"//n_i//"]","CTYPE1","LINEAR",add+,addonly-,
	        delete-,verify-,show-,update+)
            hedit(img//"[SCI,"//n_i//"]","CRVAL1",(cwave*10.),add+,addonly-,
	        delete-,verify-,show-,update+)
            hedit(img//"[SCI,"//n_i//"]","CRVAL2",1.,add+,addonly-,delete-,
	        verify-,show-,update+)
            hedit(img//"[SCI,"//n_i//"]","CRPIX1",refpix,add+,addonly-,delete-,
	        verify-,show-,update+)
            hedit(img//"[SCI,"//n_i//"]","CRPIX2",1.,add+,addonly-,delete-,
	        verify-,show-,update+)
            hedit(img//"[SCI,"//n_i//"]","CD1_1",(-10.*nmppx),add+,addonly-,
	        delete-,verify-,show-,update+)
            hedit(img//"[SCI,"//n_i//"]","CD2_2",1.0,add+,addonly-,delete-,
	        verify-,show-,update+)
            if (obsmode != "IFU") {
                hedit(img//"[SCI,"//n_i//"]","CTYPE2","LINEAR",add+,addonly-,
		    delete-,verify-,show-,update+)
                hedit(img//"[SCI,"//n_i//"]","WAT0_001","system=image",add+,
		    addonly-,delete-,verify-,show-,update+)
                hedit(img//"[SCI,"//n_i//"]","WAT2_001","wtype=linear",add+,
		    addonly-,delete-,verify-,show-,update+)
                hedit(img//"[SCI,"//n_i//"]","CD1_2",0.0,add-,addonly-,delete+,
		    verify-,show-,update+)
                hedit(img//"[SCI,"//n_i//"]","CD2_1",0.0,add-,addonly-,delete+,
		    verify-,show-,update+)
                hedit(img//"[SCI,"//n_i//"]","CD2_2",1.0,add+,addonly-,delete-,
		    verify-,show-,update+)
            }
            gemhedit(img//"[SCI,"//n_i//"]",l_key_dispaxis,l_dispaxis,"Dispersion axis")

        }
        if (imaccess(img//"[VAR,"//n_i//"]")) {
            hedit(img//"[VAR,"//n_i//"]","WAT1_001",
	        "wtype=linear label=Wavelength units=Angstroms",add-,addonly-,
		delete-,verify-,show-,update+)
            hedit(img//"[VAR,"//n_i//"]","CTYPE1","LINEAR",add+,addonly-,
	        delete-,verify-,show-,update+)
            hedit(img//"[VAR,"//n_i//"]","CRVAL1",(cwave*10.),add+,addonly-,
	        delete-,verify-,show-,update+)
            hedit(img//"[VAR,"//n_i//"]","CRPIX1",refpix,add+,addonly-,delete-,
	        verify-,show-,update+)
            hedit(img//"[VAR,"//n_i//"]","C1_1",(-10.*nmppx),add+,addonly-,
	        delete-,verify-,show-,update+)
            if (obsmode != "IFU") {
                hedit(img//"[VAR,"//n_i//"]","WAT0_001","system=image",add+,
		    addonly-,delete-,verify-,show-,update+)
                hedit(img//"[VAR,"//n_i//"]","WAT2_001","wtype=linear",add+,
		    addonly-,delete-,verify-,show-,update+)
                hedit(img//"[VAR,"//n_i//"]","CD2_1",0.0,add+,addonly-,delete-,
		    verify-,show-,update+)
                hedit(img//"[VAR,"//n_i//"]","CD2_2",1.0,add+,addonly-,delete-,
		    verify-,show-,update+)
            }
            hedit(img//"[VAR,"//n_i//"]","C2_2",1.0,add+,addonly-,delete-,
	        verify-,show-,update+)
            gemhedit(img//"[VAR,"//n_i//"]",l_key_dispaxis,l_dispaxis,"Dispersion axis")
        }
        if (imaccess(img//"[DQ,"//n_i//"]")) {
            hedit(img//"[DQ,"//n_i//"]","WAT1_001",
	        "wtype=linear label=Wavelength units=Angstroms",add-,addonly-,
		delete-,verify-,show-,update+)
            hedit(img//"[DQ,"//n_i//"]","CTYPE1","LINEAR",add+,addonly-,
	        delete-,verify-,show-,update+)
            hedit(img//"[DQ,"//n_i//"]","CRVAL1",(cwave*10.),add+,addonly-,
	        delete-,verify-,show-,update+)
            hedit(img//"[DQ,"//n_i//"]","CRPIX1",refpix,add+,addonly-,delete-,
	        verify-,show-,update+)
            hedit(img//"[DQ,"//n_i//"]","CD1_1",(-10.*nmppx),add+,addonly-,
	        delete-,verify-,show-,update+)
            if (obsmode != "IFU") {
                hedit(img//"[DQ,"//n_i//"]","WAT0_001","system=image",add+,
		    addonly-,delete-,verify-,show-,update+)
                hedit(img//"[DQ,"//n_i//"]","WAT2_001","wtype=linear",add+,
		    addonly-,delete-,verify-,show-,update+)
                hedit(img//"[DQ,"//n_i//"]","CD2_1",0.0,add+,addonly-,delete-,
		    verify-,show-,update+)
                hedit(img//"[DQ,"//n_i//"]","CD2_2",1.0,add+,addonly-,delete-,
		    verify-,show-,update+)
            }
            hedit(img//"[DQ,"//n_i//"]","CD2_2",1.0,add+,addonly-,delete-,
	        verify-,show-,update+)
            gemhedit(img//"[DQ,"//n_i//"]",l_key_dispaxis,l_dispaxis,"Dispersion axis")
        }
    }

    # final header update
    gemhedit(img//"[0]","GSAPPWAV",sdate,"Time stamp for GSAPPWAVE")
    gemhedit(img//"[0]","GEM-TLM",sdate,"Last modification with GEMINI")
}
scanfile=""
# clean up
goto clean

error:
    status=1
    goto clean

clean:
    scanfile=""
    delete(infiles,verify-, >& "dev$null")
    # close log file
    printlog(" ",l_logfile,l_verbose)
    printlog("GSAPPWAVE done",l_logfile,l_verbose)
    printlog("--------------------------------------------------------------------------------",l_logfile,l_verbose )

end
