# Copyright(c) 2004-2005 Association of Universities for Research in Astronomy, Inc.

#
# not hard coded any more define MDFEXTNAME	"MINIMAL.TAB"
#

# This file contains:
#         gdpropagate(...) - 
#
# Support routines:
#         gdpropagate(...) - Short description
#
# See also:
#         gdpropagate(...) - Short description

#
# ... define statements ...
#

# GDPROPAGATE --
#	This routine checks to see if key files have been propagated based
# 	on Gemini MEF Data Type.  (WARNING: Data Type Currently
# 	Ignored... propagates 	MDF, aka MINIMAL.TAB extensions)
# NOTE:: TODO:: I want to put PHU propagation in here!
include "mefio.h"
include <error.h>


procedure gdpropagate (source, dest, tname)
char source[ARB]
char dest[ARB]
char tname[ARB] # # locals below #
char mdf_ext[SZ_LINE]
pointer smep	# for mefio memap
pointer smef, dmef # for fitsutil manipulations
int	mdfind		# mdf index (aka mdf "group" number)
bool ldebug
pointer mef_open()
int meindbnv()
pointer memap()
char tmpstr[SZ_LINE]
int errc
bool verbose
bool clgetb()
int errget()

errchk memap(),meindvnv(),mef_open(), mef_copy_extn(), mef_close(), meunmap() 

begin
    
	ldebug = false # true

    if (ldebug) {
        call printf("gdpropagate(%s,%s,%s)\n")
        call pargstr(source)
        call pargstr(dest)
        call pargstr(tname)
        call flush(STDOUT)
    }
    
    call clgstr("mdf_ext", mdf_ext, SZ_LINE)
    verbose= clgetb("verbose")
# memap
    
    iferr(smep = memap(source)){
        errc = errget(tmpstr, SZ_LINE)
        call task_err(errc, tmpstr)
        call sprintf(tmpstr, SZ_LINE, "gdpropagate(): cannot copy MDF,\"%s\"\nIs this is a correct reference image (not a constant)")
        call pargstr(source)
        call log_err(MEERR_MEMAPFAIL, tmpstr)
        call erract(EA_ERROR)
    }
    
    if (ldebug){
        call printf("smep = %x\n")
        call pargi(smep)
        call flush(STDOUT)
        call meprint(smep)
    }
# getindex by name and ver (get just one (NOTE::is that ok?))
	mdfind = meindbnv(smep, mdf_ext, -1)

	if (mdfind > 0)
	{
		if (verbose) {
			call sprintf(tmpstr, SZ_LINE, "Going to copy MDF from %s[%d]")
			call pargstr(source)
			call pargi(mdfind)
            call log_info(tmpstr)
		}
	}
	else
	{
        if (verbose){
		    call log_info("No MDF present in source");
        }
		return
	}

# use fitsutil open source
	smef = mef_open(source, READ_ONLY, 0)
	
# use fitsutil open dest
	dmef = mef_open(dest, APPEND, 0)
	
# use fitsutil to copy source to dest
	call mef_copy_extn (smef, dmef, mdfind)
	
# use fitsutil close dest
	call mef_close(dmef)
	
# use fitsutil close source
	call mef_close(smef)

	call meunmap(smep)
end
